# pylint:disable=line-too-long
from __future__ import annotations
import logging
from collections import OrderedDict

from angr.sim_type import SimTypeFunction, SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat, SimTypePointer, SimTypeChar, SimStruct, SimTypeArray, SimTypeBottom, SimUnion, SimTypeBool, SimTypeRef
from angr.calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from angr.procedures import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.type_collection_names = ["win32"]
lib.set_default_cc("X86", SimCCStdcall)
lib.set_default_cc("AMD64", SimCCMicrosoftAMD64)
lib.set_library_names("wsmsvc.dll")
prototypes = \
    {
        #
        'WSManInitialize': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["flags", "apiHandle"]),
        #
        'WSManDeinitialize': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["apiHandle", "flags"]),
        #
        'WSManGetErrorMessage': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["apiHandle", "flags", "languageCode", "errorCode", "messageLength", "message", "messageLengthUsed"]),
        #
        'WSManCreateSession': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSMAN_AUTHENTICATION_CREDENTIALS", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_PROXY_INFO", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["apiHandle", "connection", "flags", "serverAuthenticationCredentials", "proxyInfo", "session"]),
        #
        'WSManCloseSession': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["session", "flags"]),
        #
        'WSManSetSessionOption': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="WSManSessionOption"), SimTypePointer(SimTypeRef("WSMAN_DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["session", "option", "data"]),
        #
        'WSManGetSessionOptionAsDword': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="WSManSessionOption"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["session", "option", "value"]),
        #
        'WSManGetSessionOptionAsString': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="WSManSessionOption"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["session", "option", "stringLength", "string", "stringLengthUsed"]),
        #
        'WSManCloseOperation': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["operationHandle", "flags"]),
        #
        'WSManCreateShell': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("WSMAN_SHELL_STARTUP_INFO_V11", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_OPTION_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_DATA", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["session", "flags", "resourceUri", "startupInfo", "options", "createXml", "async", "shell"]),
        #
        'WSManRunShellCommand': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("WSMAN_COMMAND_ARG_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_OPTION_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["shell", "flags", "commandLine", "args", "options", "async", "command"]),
        #
        'WSManSignalShell': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["shell", "command", "flags", "code", "async", "signalOperation"]),
        #
        'WSManReceiveShellOutput': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSMAN_STREAM_ID_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["shell", "command", "flags", "desiredStreamSet", "async", "receiveOperation"]),
        #
        'WSManSendShellInput': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("WSMAN_DATA", SimStruct), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["shell", "command", "flags", "streamId", "streamData", "endOfStream", "async", "sendOperation"]),
        #
        'WSManCloseCommand': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["commandHandle", "flags", "async"]),
        #
        'WSManCloseShell': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["shellHandle", "flags", "async"]),
        #
        'WSManCreateShellEx': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("WSMAN_SHELL_STARTUP_INFO_V11", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_OPTION_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_DATA", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["session", "flags", "resourceUri", "shellId", "startupInfo", "options", "createXml", "async", "shell"]),
        #
        'WSManRunShellCommandEx': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("WSMAN_COMMAND_ARG_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_OPTION_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["shell", "flags", "commandId", "commandLine", "args", "options", "async", "command"]),
        #
        'WSManDisconnectShell': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSMAN_SHELL_DISCONNECT_INFO", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["shell", "flags", "disconnectInfo", "async"]),
        #
        'WSManReconnectShell': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["shell", "flags", "async"]),
        #
        'WSManReconnectShellCommand': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0)], SimTypeBottom(label="Void"), arg_names=["commandHandle", "flags", "async"]),
        #
        'WSManConnectShell': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("WSMAN_OPTION_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_DATA", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["session", "flags", "resourceUri", "shellID", "options", "connectXml", "async", "shell"]),
        #
        'WSManConnectShellCommand': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("WSMAN_OPTION_SET", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_DATA", SimStruct), offset=0), SimTypePointer(SimTypeRef("WSMAN_SHELL_ASYNC", SimStruct), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeBottom(label="Void"), arg_names=["shell", "flags", "commandID", "options", "connectXml", "async", "command"]),
        #
        'WSManPluginReportContext': SimTypeFunction([SimTypePointer(SimTypeRef("WSMAN_PLUGIN_REQUEST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["requestDetails", "flags", "context"]),
        #
        'WSManPluginReceiveResult': SimTypeFunction([SimTypePointer(SimTypeRef("WSMAN_PLUGIN_REQUEST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("WSMAN_DATA", SimStruct), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["requestDetails", "flags", "stream", "streamResult", "commandState", "exitCode"]),
        #
        'WSManPluginOperationComplete': SimTypeFunction([SimTypePointer(SimTypeRef("WSMAN_PLUGIN_REQUEST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["requestDetails", "flags", "errorCode", "extendedInformation"]),
        #
        'WSManPluginGetOperationParameters': SimTypeFunction([SimTypePointer(SimTypeRef("WSMAN_PLUGIN_REQUEST", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSMAN_DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["requestDetails", "flags", "data"]),
        #
        'WSManPluginGetConfiguration': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSMAN_DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pluginContext", "flags", "data"]),
        #
        'WSManPluginReportCompletion': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["pluginContext", "flags"]),
        #
        'WSManPluginFreeRequestDetails': SimTypeFunction([SimTypePointer(SimTypeRef("WSMAN_PLUGIN_REQUEST", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["requestDetails"]),
        #
        'WSManPluginAuthzUserComplete': SimTypeFunction([SimTypePointer(SimTypeRef("WSMAN_SENDER_DETAILS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["senderDetails", "flags", "userAuthorizationContext", "impersonationToken", "userIsAdministrator", "errorCode", "extendedErrorInformation"]),
        #
        'WSManPluginAuthzOperationComplete': SimTypeFunction([SimTypePointer(SimTypeRef("WSMAN_SENDER_DETAILS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["senderDetails", "flags", "userAuthorizationContext", "errorCode", "extendedErrorInformation"]),
        #
        'WSManPluginAuthzQueryQuotaComplete': SimTypeFunction([SimTypePointer(SimTypeRef("WSMAN_SENDER_DETAILS", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("WSMAN_AUTHZ_QUOTA", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["senderDetails", "flags", "quota", "errorCode", "extendedErrorInformation"]),
    }

lib.set_prototypes(prototypes)



# pylint:disable=line-too-long
from __future__ import annotations
import logging
from collections import OrderedDict

from angr.sim_type import SimTypeFunction, SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat, SimTypePointer, SimTypeChar, SimStruct, SimTypeArray, SimTypeBottom, SimUnion, SimTypeBool, SimTypeRef
from angr.calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from angr.procedures import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.type_collection_names = ["win32"]
lib.set_default_cc("X86", SimCCStdcall)
lib.set_default_cc("AMD64", SimCCMicrosoftAMD64)
lib.set_library_names("httpapi.dll")
prototypes = \
    {
        #
        'HttpInitialize': SimTypeFunction([SimTypeRef("HTTPAPI_VERSION", SimStruct), SimTypeInt(signed=False, label="HTTP_INITIALIZE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Version", "Flags", "pReserved"]),
        #
        'HttpTerminate': SimTypeFunction([SimTypeInt(signed=False, label="HTTP_INITIALIZE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Flags", "pReserved"]),
        #
        'HttpCreateHttpHandle': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "Reserved"]),
        #
        'HttpCreateRequestQueue': SimTypeFunction([SimTypeRef("HTTPAPI_VERSION", SimStruct), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("SECURITY_ATTRIBUTES", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Version", "Name", "SecurityAttributes", "Flags", "RequestQueueHandle"]),
        #
        'HttpCloseRequestQueue': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle"]),
        #
        'HttpSetRequestQueueProperty': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="HTTP_SERVER_PROPERTY"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "Property", "PropertyInformation", "PropertyInformationLength", "Reserved1", "Reserved2"]),
        #
        'HttpQueryRequestQueueProperty': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="HTTP_SERVER_PROPERTY"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "Property", "PropertyInformation", "PropertyInformationLength", "Reserved1", "ReturnLength", "Reserved2"]),
        #
        'HttpSetRequestProperty': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="HTTP_REQUEST_PROPERTY"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "Id", "PropertyId", "Input", "InputPropertySize", "Overlapped"]),
        #
        'HttpShutdownRequestQueue': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle"]),
        #
        'HttpReceiveClientCertificate': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("HTTP_SSL_CLIENT_CERT_INFO", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "ConnectionId", "Flags", "SslClientCertInfo", "SslClientCertInfoSize", "BytesReceived", "Overlapped"]),
        #
        'HttpCreateServerSession': SimTypeFunction([SimTypeRef("HTTPAPI_VERSION", SimStruct), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Version", "ServerSessionId", "Reserved"]),
        #
        'HttpCloseServerSession': SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ServerSessionId"]),
        #
        'HttpQueryServerSessionProperty': SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="HTTP_SERVER_PROPERTY"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ServerSessionId", "Property", "PropertyInformation", "PropertyInformationLength", "ReturnLength"]),
        #
        'HttpSetServerSessionProperty': SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="HTTP_SERVER_PROPERTY"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ServerSessionId", "Property", "PropertyInformation", "PropertyInformationLength"]),
        #
        'HttpAddUrl': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "FullyQualifiedUrl", "Reserved"]),
        #
        'HttpRemoveUrl': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "FullyQualifiedUrl"]),
        #
        'HttpCreateUrlGroup': SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["ServerSessionId", "pUrlGroupId", "Reserved"]),
        #
        'HttpCloseUrlGroup': SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=False, label="UInt32"), arg_names=["UrlGroupId"]),
        #
        'HttpAddUrlToUrlGroup': SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["UrlGroupId", "pFullyQualifiedUrl", "UrlContext", "Reserved"]),
        #
        'HttpRemoveUrlFromUrlGroup': SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["UrlGroupId", "pFullyQualifiedUrl", "Flags"]),
        #
        'HttpSetUrlGroupProperty': SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="HTTP_SERVER_PROPERTY"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["UrlGroupId", "Property", "PropertyInformation", "PropertyInformationLength"]),
        #
        'HttpQueryUrlGroupProperty': SimTypeFunction([SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="HTTP_SERVER_PROPERTY"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["UrlGroupId", "Property", "PropertyInformation", "PropertyInformationLength", "ReturnLength"]),
        #
        'HttpPrepareUrl': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Reserved", "Flags", "Url", "PreparedUrl"]),
        #
        'HttpReceiveHttpRequest': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="HTTP_RECEIVE_HTTP_REQUEST_FLAGS"), SimTypePointer(SimTypeRef("HTTP_REQUEST_V2", SimStruct), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "RequestId", "Flags", "RequestBuffer", "RequestBufferLength", "BytesReturned", "Overlapped"]),
        #
        'HttpReceiveRequestEntityBody': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "RequestId", "Flags", "EntityBuffer", "EntityBufferLength", "BytesReturned", "Overlapped"]),
        #
        'HttpSendHttpResponse': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("HTTP_RESPONSE_V2", SimStruct), offset=0), SimTypePointer(SimTypeRef("HTTP_CACHE_POLICY", SimStruct), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeRef("HTTP_LOG_DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "RequestId", "Flags", "HttpResponse", "CachePolicy", "BytesSent", "Reserved1", "Reserved2", "Overlapped", "LogData"]),
        #
        'HttpSendResponseEntityBody': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeRef("HTTP_DATA_CHUNK", SimStruct), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0), SimTypePointer(SimTypeRef("HTTP_LOG_DATA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "RequestId", "Flags", "EntityChunkCount", "EntityChunks", "BytesSent", "Reserved1", "Reserved2", "Overlapped", "LogData"]),
        #
        'HttpDeclarePush': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="HTTP_VERB"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeRef("HTTP_REQUEST_HEADERS", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "RequestId", "Verb", "Path", "Query", "Headers"]),
        #
        'HttpWaitForDisconnect': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "ConnectionId", "Overlapped"]),
        #
        'HttpWaitForDisconnectEx': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "ConnectionId", "Reserved", "Overlapped"]),
        #
        'HttpCancelHttpRequest': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "RequestId", "Overlapped"]),
        #
        'HttpWaitForDemandStart': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "Overlapped"]),
        #
        'HttpIsFeatureSupported': SimTypeFunction([SimTypeInt(signed=False, label="HTTP_FEATURE_ID")], SimTypeInt(signed=True, label="Int32"), arg_names=["FeatureId"]),
        #
        'HttpDelegateRequestEx': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("HTTP_DELEGATE_REQUEST_PROPERTY_INFO", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "DelegateQueueHandle", "RequestId", "DelegateUrlGroupId", "PropertyInfoSetSize", "PropertyInfoSet"]),
        #
        'HttpFindUrlGroupId': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["FullyQualifiedUrl", "RequestQueueHandle", "UrlGroupId"]),
        #
        'HttpFlushResponseCache': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "UrlPrefix", "Flags", "Overlapped"]),
        #
        'HttpAddFragmentToCache': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("HTTP_DATA_CHUNK", SimStruct), offset=0), SimTypePointer(SimTypeRef("HTTP_CACHE_POLICY", SimStruct), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "UrlPrefix", "DataChunk", "CachePolicy", "Overlapped"]),
        #
        'HttpReadFragmentFromCache': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeRef("HTTP_BYTE_RANGE", SimStruct), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["RequestQueueHandle", "UrlPrefix", "ByteRange", "Buffer", "BufferLength", "BytesRead", "Overlapped"]),
        #
        'HttpSetServiceConfiguration': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_ID"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ServiceHandle", "ConfigId", "pConfigInformation", "ConfigInformationLength", "pOverlapped"]),
        #
        'HttpUpdateServiceConfiguration': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_ID"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Handle", "ConfigId", "ConfigInfo", "ConfigInfoLength", "Overlapped"]),
        #
        'HttpDeleteServiceConfiguration': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_ID"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ServiceHandle", "ConfigId", "pConfigInformation", "ConfigInformationLength", "pOverlapped"]),
        #
        'HttpQueryServiceConfiguration': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="HTTP_SERVICE_CONFIG_ID"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeRef("OVERLAPPED", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ServiceHandle", "ConfigId", "pInput", "InputLength", "pOutput", "OutputLength", "pReturnLength", "pOverlapped"]),
        #
        'HttpGetExtension': SimTypeFunction([SimTypeRef("HTTPAPI_VERSION", SimStruct), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["Version", "Extension", "Buffer", "BufferSize"]),
    }

lib.set_prototypes(prototypes)



from __future__ import annotations
from .preset import DecompilationPreset
from angr.analyses.decompiler.optimization_passes import (
    RegisterSaveAreaSimplifier,
    StackCanarySimplifier,
    WinStackCanarySimplifier,
    BasePointerSaveSimplifier,
    ConstantDereferencesSimplifier,
    RetAddrSaveSimplifier,
    X86GccGetPcSimplifier,
    CallStatementRewriter,
    SwitchReusedEntryRewriter,
)


preset_basic = DecompilationPreset(
    "basic",
    [
        RegisterSaveAreaSimplifier,
        StackCanarySimplifier,
        WinStackCanarySimplifier,
        BasePointerSaveSimplifier,
        ConstantDereferencesSimplifier,
        RetAddrSaveSimplifier,
        X86GccGetPcSimplifier,
        CallStatementRewriter,
        SwitchReusedEntryRewriter,
    ],
)


__all__ = ["preset_basic"]



from __future__ import annotations
from enum import Enum, unique


_ansi_prefix = "\x1b["


#
# Ansi colors
#


clear: str = f"{_ansi_prefix}0m"


@unique
class Color(Enum):
    """
    The basic ansi colors
    """

    black = 30
    red = 31
    green = 32
    yellow = 33
    blue = 34
    magenta = 35
    cyan = 36
    white = 37


BackgroundColor = unique(Enum("BackgroundColor", {i.name: (i.value + 10) for i in Color}))


#
# Functions
#


def color(c: Color | BackgroundColor, bright: bool):
    """
    Return the ansi prefix using the given code
    Bright may not be used with a BackgroundColor
    """
    if bright and isinstance(c, BackgroundColor):
        raise ValueError("Backgrounds should not be bright")
    return f"{_ansi_prefix}{c.value};1m" if bright else f"{_ansi_prefix}{c.value}m"



# pylint:disable=line-too-long
from __future__ import annotations
import logging
from collections import OrderedDict

from angr.sim_type import SimTypeFunction, SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat, SimTypePointer, SimTypeChar, SimStruct, SimTypeArray, SimTypeBottom, SimUnion, SimTypeBool, SimTypeRef
from angr.calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from angr.procedures import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.type_collection_names = ["win32"]
lib.set_default_cc("X86", SimCCStdcall)
lib.set_default_cc("AMD64", SimCCMicrosoftAMD64)
lib.set_library_names("oledlg.dll")
prototypes = \
    {
        #
        'OleUIAddVerbMenuW': SimTypeFunction([SimTypeBottom(label="IOleObject"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpOleObj", "lpszShortType", "hMenu", "uPos", "uIDVerbMin", "uIDVerbMax", "bAddConvert", "idConvert", "lphMenu"]),
        #
        'OleUIAddVerbMenuA': SimTypeFunction([SimTypeBottom(label="IOleObject"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpOleObj", "lpszShortType", "hMenu", "uPos", "uIDVerbMin", "uIDVerbMax", "bAddConvert", "idConvert", "lphMenu"]),
        #
        'OleUIInsertObjectW': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUIINSERTOBJECTW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIInsertObjectA': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUIINSERTOBJECTA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIPasteSpecialW': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUIPASTESPECIALW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIPasteSpecialA': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUIPASTESPECIALA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIEditLinksW': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUIEDITLINKSW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIEditLinksA': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUIEDITLINKSA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIChangeIconW': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUICHANGEICONW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIChangeIconA': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUICHANGEICONA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIConvertW': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUICONVERTW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIConvertA': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUICONVERTA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUICanConvertOrActivateAs': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeShort(signed=False, label="UInt16")], SimTypeInt(signed=True, label="Int32"), arg_names=["rClsid", "fIsLinkedObject", "wFormat"]),
        #
        'OleUIBusyW': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUIBUSYW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIBusyA': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUIBUSYA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIChangeSourceW': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUICHANGESOURCEW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIChangeSourceA': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUICHANGESOURCEA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIObjectPropertiesW': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUIOBJECTPROPSW", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIObjectPropertiesA': SimTypeFunction([SimTypePointer(SimTypeRef("OLEUIOBJECTPROPSA", SimStruct), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["param0"]),
        #
        'OleUIPromptUserW': SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["nTemplate", "hwndParent"]),
        #
        'OleUIPromptUserA': SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["nTemplate", "hwndParent"]),
        #
        'OleUIUpdateLinksW': SimTypeFunction([SimTypeBottom(label="IOleUILinkContainerW"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpOleUILinkCntr", "hwndParent", "lpszTitle", "cLinks"]),
        #
        'OleUIUpdateLinksA': SimTypeFunction([SimTypeBottom(label="IOleUILinkContainerA"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpOleUILinkCntr", "hwndParent", "lpszTitle", "cLinks"]),
    }

lib.set_prototypes(prototypes)
